@@ -6,10 +6,10 @@
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions: 
+ * Software is furnished to do so, subject to the following conditions:
  *
  * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software. 
+ * in all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -30,6 +30,7 @@
 #include "bases.h"
 #include "char.h"
 #include "macros.h"
+#include "unicodeset.h"
 
 DECLARE_CONSTANTS_TYPE(UProperty);
 DECLARE_CONSTANTS_TYPE(UCharDirection);
@@ -37,12 +38,23 @@ DECLARE_CONSTANTS_TYPE(UCharCategory);
 DECLARE_CONSTANTS_TYPE(UBlockCode);
 DECLARE_CONSTANTS_TYPE(UCharNameChoice);
 DECLARE_CONSTANTS_TYPE(UPropertyNameChoice);
+DECLARE_CONSTANTS_TYPE(UWordBreakValues);
+DECLARE_CONSTANTS_TYPE(UJoiningGroup);
+DECLARE_CONSTANTS_TYPE(ULineBreak);
+DECLARE_CONSTANTS_TYPE(UGraphemeClusterBreak);
+DECLARE_CONSTANTS_TYPE(UHangulSyllableType);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(52, 0, 0)
+DECLARE_CONSTANTS_TYPE(UBidiPairedBracketType);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+DECLARE_CONSTANTS_TYPE(UIndicPositionalCategory);
+DECLARE_CONSTANTS_TYPE(UIndicSyllabicCategory);
+DECLARE_CONSTANTS_TYPE(UVerticalOrientation);
+#endif
+
 
 /* Char */
 
-struct UNone;
-typedef struct UNone UNone;
-
 class t_char : public _wrapper {
 public:
     UNone *object;
@@ -53,6 +65,9 @@ typedef UChar32 (*uchar32_char_fn)(UChar32 c);
 
 static int t_char_init(t_char *self, PyObject *args, PyObject *kwds);
 static PyObject *t_char_hasBinaryProperty(PyTypeObject *type, PyObject *args);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+static PyObject *t_char_getBinaryPropertySet(PyTypeObject *type, PyObject *arg);
+#endif
 static PyObject *t_char_getIntPropertyValue(PyTypeObject *type, PyObject *args);
 static PyObject *t_char_getIntPropertyMinValue(PyTypeObject *type,
                                                PyObject *arg);
@@ -98,6 +113,7 @@ static PyObject *t_char_enumCharNames(PyTypeObject *type, PyObject *args);
 static PyObject *t_char_getPropertyName(PyTypeObject *type, PyObject *args);
 static PyObject *t_char_getPropertyEnum(PyTypeObject *type, PyObject *arg);
 static PyObject *t_char_isIDStart(PyTypeObject *type, PyObject *arg);
+static PyObject *t_char_isIDPart(PyTypeObject *type, PyObject *arg);
 static PyObject *t_char_isIDIgnorable(PyTypeObject *type, PyObject *arg);
 static PyObject *t_char_isJavaIDStart(PyTypeObject *type, PyObject *arg);
 static PyObject *t_char_isJavaIDPart(PyTypeObject *type, PyObject *arg);
@@ -113,6 +129,9 @@ static PyObject *t_char_getFC_NFKC_Closure(PyTypeObject *type, PyObject *arg);
 
 static PyMethodDef t_char_methods[] = {
     DECLARE_METHOD(t_char, hasBinaryProperty, METH_VARARGS | METH_CLASS),
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    DECLARE_METHOD(t_char, getBinaryPropertySet, METH_O | METH_CLASS),
+#endif
     DECLARE_METHOD(t_char, getIntPropertyValue, METH_VARARGS | METH_CLASS),
     DECLARE_METHOD(t_char, getIntPropertyMinValue, METH_O | METH_CLASS),
     DECLARE_METHOD(t_char, getIntPropertyMaxValue, METH_O | METH_CLASS),
@@ -156,6 +175,7 @@ static PyMethodDef t_char_methods[] = {
     DECLARE_METHOD(t_char, getPropertyName, METH_VARARGS | METH_CLASS),
     DECLARE_METHOD(t_char, getPropertyEnum, METH_O | METH_CLASS),
     DECLARE_METHOD(t_char, isIDStart, METH_O | METH_CLASS),
+    DECLARE_METHOD(t_char, isIDPart, METH_O | METH_CLASS),
     DECLARE_METHOD(t_char, isIDIgnorable, METH_O | METH_CLASS),
     DECLARE_METHOD(t_char, isJavaIDStart, METH_O | METH_CLASS),
     DECLARE_METHOD(t_char, isJavaIDPart, METH_O | METH_CLASS),
@@ -211,6 +231,24 @@ static PyObject *t_char_hasBinaryProperty(PyTypeObject *type, PyObject *args)
     return PyErr_SetArgsError((PyObject *) type, "hasBinaryProperty", args);
 }
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+static PyObject *t_char_getBinaryPropertySet(PyTypeObject *type, PyObject *arg)
+{
+    UProperty prop;
+
+    if (!parseArg(arg, "i", &prop))
+    {
+        const USet *set;
+        STATUS_CALL(set = u_getBinaryPropertySet(prop, &status));
+
+        return wrap_UnicodeSet(
+            const_cast<UnicodeSet *>(UnicodeSet::fromUSet(set)), 0);
+    }
+
+    return PyErr_SetArgsError((PyObject *) type, "getBinaryPropertySet", arg);
+}
+#endif
+
 static PyObject *t_char_getIntPropertyValue(PyTypeObject *type, PyObject *args)
 {
     UnicodeString *u, _u;
@@ -380,6 +418,10 @@ static PyObject *t_char_isIDStart(PyTypeObject *type, PyObject *arg) {
     return t_char_fn(u_isIDStart, "isIDStart", type, arg);
 }
 
+static PyObject *t_char_isIDPart(PyTypeObject *type, PyObject *arg) {
+    return t_char_fn(u_isIDPart, "isIDPart", type, arg);
+}
+
 static PyObject *t_char_isIDIgnorable(PyTypeObject *type, PyObject *arg) {
     return t_char_fn(u_isIDIgnorable, "isIDIgnorable", type, arg);
 }
@@ -680,7 +722,7 @@ static PyObject *t_char_fn(uchar32_char_fn fn, const char *name,
     {
         UnicodeString result;
         result += (*fn)(u->char32At(0));
-        
+
         return PyUnicode_FromUnicodeString(&result);
     }
 
@@ -728,7 +770,7 @@ static PyObject *t_char_foldCase(PyTypeObject *type, PyObject *args)
         {
             UnicodeString result;
             result += u_foldCase(u->char32At(0), U_FOLD_CASE_DEFAULT);
-        
+
             return PyUnicode_FromUnicodeString(&result);
         }
         break;
@@ -739,7 +781,7 @@ static PyObject *t_char_foldCase(PyTypeObject *type, PyObject *args)
         {
             UnicodeString result;
             result += u_foldCase(u->char32At(0), options);
-        
+
             return PyUnicode_FromUnicodeString(&result);
         }
         break;
@@ -851,6 +893,19 @@ void _init_char(PyObject *m)
     INSTALL_CONSTANTS_TYPE(UBlockCode, m);
     INSTALL_CONSTANTS_TYPE(UCharNameChoice, m);
     INSTALL_CONSTANTS_TYPE(UPropertyNameChoice, m);
+    INSTALL_CONSTANTS_TYPE(UWordBreakValues, m);
+    INSTALL_CONSTANTS_TYPE(UJoiningGroup, m);
+    INSTALL_CONSTANTS_TYPE(ULineBreak, m);
+    INSTALL_CONSTANTS_TYPE(UGraphemeClusterBreak, m);
+    INSTALL_CONSTANTS_TYPE(UHangulSyllableType, m);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(52, 0, 0)
+    INSTALL_CONSTANTS_TYPE(UBidiPairedBracketType, m);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    INSTALL_CONSTANTS_TYPE(UIndicPositionalCategory, m);
+    INSTALL_CONSTANTS_TYPE(UIndicSyllabicCategory, m);
+    INSTALL_CONSTANTS_TYPE(UVerticalOrientation, m);
+#endif
     INSTALL_STRUCT(Char, m);
 
     INSTALL_ENUM(Char, "FOLD_CASE_DEFAULT", U_FOLD_CASE_DEFAULT);
@@ -914,7 +969,6 @@ void _init_char(PyObject *m)
     INSTALL_ENUM(UProperty, "CHANGES_WHEN_CASEMAPPED", UCHAR_CHANGES_WHEN_CASEMAPPED);
     INSTALL_ENUM(UProperty, "CHANGES_WHEN_NFKC_CASEFOLDED", UCHAR_CHANGES_WHEN_NFKC_CASEFOLDED);
 #endif
-    INSTALL_ENUM(UProperty, "BINARY_LIMIT", UCHAR_BINARY_LIMIT);
     INSTALL_ENUM(UProperty, "BIDI_CLASS", UCHAR_BIDI_CLASS);
     INSTALL_ENUM(UProperty, "INT_START", UCHAR_INT_START);
     INSTALL_ENUM(UProperty, "BLOCK", UCHAR_BLOCK);
@@ -940,13 +994,10 @@ void _init_char(PyObject *m)
 #if U_ICU_VERSION_HEX >= VERSION_HEX(52, 0, 0)
     INSTALL_ENUM(UProperty, "BIDI_PAIRED_BRACKET_TYPE", UCHAR_BIDI_PAIRED_BRACKET_TYPE);
 #endif
-    INSTALL_ENUM(UProperty, "INT_LIMIT", UCHAR_INT_LIMIT);
     INSTALL_ENUM(UProperty, "GENERAL_CATEGORY_MASK", UCHAR_GENERAL_CATEGORY_MASK);
     INSTALL_ENUM(UProperty, "MASK_START", UCHAR_MASK_START);
-    INSTALL_ENUM(UProperty, "MASK_LIMIT", UCHAR_MASK_LIMIT);
     INSTALL_ENUM(UProperty, "NUMERIC_VALUE", UCHAR_NUMERIC_VALUE);
     INSTALL_ENUM(UProperty, "DOUBLE_START", UCHAR_DOUBLE_START);
-    INSTALL_ENUM(UProperty, "DOUBLE_LIMIT", UCHAR_DOUBLE_LIMIT);
     INSTALL_ENUM(UProperty, "AGE", UCHAR_AGE);
     INSTALL_ENUM(UProperty, "STRING_START", UCHAR_STRING_START);
     INSTALL_ENUM(UProperty, "BIDI_MIRRORING_GLYPH", UCHAR_BIDI_MIRRORING_GLYPH);
@@ -962,11 +1013,29 @@ void _init_char(PyObject *m)
 #if U_ICU_VERSION_HEX >= VERSION_HEX(52, 0, 0)
     INSTALL_ENUM(UProperty, "BIDI_PAIRED_BRACKET", UCHAR_BIDI_PAIRED_BRACKET);
 #endif
-    INSTALL_ENUM(UProperty, "STRING_LIMIT", UCHAR_STRING_LIMIT);
 #if U_ICU_VERSION_HEX >= 0x04060000
     INSTALL_ENUM(UProperty, "SCRIPT_EXTENSIONS", UCHAR_SCRIPT_EXTENSIONS);
     INSTALL_ENUM(UProperty, "OTHER_PROPERTY_START", UCHAR_OTHER_PROPERTY_START);
-    INSTALL_ENUM(UProperty, "OTHER_PROPERTY_LIMIT", UCHAR_OTHER_PROPERTY_LIMIT);
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(57, 0, 0)
+    INSTALL_ENUM(UProperty, "EMOJI", UCHAR_EMOJI);
+    INSTALL_ENUM(UProperty, "EMOJI_PRESENTATION", UCHAR_EMOJI_PRESENTATION);
+    INSTALL_ENUM(UProperty, "EMOJI_MODIFIER", UCHAR_EMOJI_MODIFIER);
+    INSTALL_ENUM(UProperty, "EMOJI_MODIFIER_BASE", UCHAR_EMOJI_MODIFIER_BASE);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(60, 0, 0)
+    INSTALL_ENUM(UProperty, "EMOJI_COMPONENT", UCHAR_EMOJI_COMPONENT);
+    INSTALL_ENUM(UProperty, "REGIONAL_INDICATOR", UCHAR_REGIONAL_INDICATOR);
+    INSTALL_ENUM(UProperty, "PREPENDED_CONCATENATION_MARK", UCHAR_PREPENDED_CONCATENATION_MARK);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(62, 0, 0)
+    INSTALL_ENUM(UProperty, "EXTENDED_PICTOGRAPHIC", UCHAR_EXTENDED_PICTOGRAPHIC);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    INSTALL_ENUM(UProperty, "INDIC_POSITIONAL_CATEGORY", UCHAR_INDIC_POSITIONAL_CATEGORY);
+    INSTALL_ENUM(UProperty, "INDIC_SYLLABIC_CATEGORY", UCHAR_INDIC_SYLLABIC_CATEGORY);
+    INSTALL_ENUM(UProperty, "VERTICAL_ORIENTATION", UCHAR_VERTICAL_ORIENTATION);
 #endif
     INSTALL_ENUM(UProperty, "INVALID_CODE ", UCHAR_INVALID_CODE );
 
@@ -1292,6 +1361,53 @@ void _init_char(PyObject *m)
     INSTALL_ENUM(UBlockCode, "SUPPLEMENTAL_ARROWS_C", UBLOCK_SUPPLEMENTAL_ARROWS_C);
     INSTALL_ENUM(UBlockCode, "TIRHUTA", UBLOCK_TIRHUTA);
     INSTALL_ENUM(UBlockCode, "WARANG_CITI", UBLOCK_WARANG_CITI);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(56, 0, 0)
+    INSTALL_ENUM(UBlockCode, "AHOM", UBLOCK_AHOM);
+    INSTALL_ENUM(UBlockCode, "ANATOLIAN_HIEROGLYPHS", UBLOCK_ANATOLIAN_HIEROGLYPHS);
+    INSTALL_ENUM(UBlockCode, "CHEROKEE_SUPPLEMENT", UBLOCK_CHEROKEE_SUPPLEMENT);
+    INSTALL_ENUM(UBlockCode, "CJK_UNIFIED_IDEOGRAPHS_EXTENSION_E", UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_E);
+    INSTALL_ENUM(UBlockCode, "EARLY_DYNASTIC_CUNEIFORM", UBLOCK_EARLY_DYNASTIC_CUNEIFORM);
+    INSTALL_ENUM(UBlockCode, "HATRAN", UBLOCK_HATRAN);
+    INSTALL_ENUM(UBlockCode, "MULTANI", UBLOCK_MULTANI);
+    INSTALL_ENUM(UBlockCode, "OLD_HUNGARIAN", UBLOCK_OLD_HUNGARIAN);
+    INSTALL_ENUM(UBlockCode, "SUPPLEMENTAL_SYMBOLS_AND_PICTOGRAPHS", UBLOCK_SUPPLEMENTAL_SYMBOLS_AND_PICTOGRAPHS);
+    INSTALL_ENUM(UBlockCode, "SUTTON_SIGNWRITING", UBLOCK_SUTTON_SIGNWRITING);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(58, 0, 0)
+    INSTALL_ENUM(UBlockCode, "ADLAM", UBLOCK_ADLAM);
+    INSTALL_ENUM(UBlockCode, "BHAIKSUKI", UBLOCK_BHAIKSUKI);
+    INSTALL_ENUM(UBlockCode, "CYRILLIC_EXTENDED_C", UBLOCK_CYRILLIC_EXTENDED_C);
+    INSTALL_ENUM(UBlockCode, "GLAGOLITIC_SUPPLEMENT", UBLOCK_GLAGOLITIC_SUPPLEMENT);
+    INSTALL_ENUM(UBlockCode, "IDEOGRAPHIC_SYMBOLS_AND_PUNCTUATION", UBLOCK_IDEOGRAPHIC_SYMBOLS_AND_PUNCTUATION);
+    INSTALL_ENUM(UBlockCode, "MARCHEN", UBLOCK_MARCHEN);
+    INSTALL_ENUM(UBlockCode, "MONGOLIAN_SUPPLEMENT", UBLOCK_MONGOLIAN_SUPPLEMENT);
+    INSTALL_ENUM(UBlockCode, "NEWA", UBLOCK_NEWA);
+    INSTALL_ENUM(UBlockCode, "OSAGE", UBLOCK_OSAGE);
+    INSTALL_ENUM(UBlockCode, "TANGUT", UBLOCK_TANGUT);
+    INSTALL_ENUM(UBlockCode, "TANGUT_COMPONENTS", UBLOCK_TANGUT_COMPONENTS);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(60, 0, 0)
+    INSTALL_ENUM(UBlockCode, "CJK_UNIFIED_IDEOGRAPHS_EXTENSION_F", UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_F);
+    INSTALL_ENUM(UBlockCode, "KANA_EXTENDED_A", UBLOCK_KANA_EXTENDED_A);
+    INSTALL_ENUM(UBlockCode, "MASARAM_GONDI", UBLOCK_MASARAM_GONDI);
+    INSTALL_ENUM(UBlockCode, "NUSHU", UBLOCK_NUSHU);
+    INSTALL_ENUM(UBlockCode, "SOYOMBO", UBLOCK_SOYOMBO);
+    INSTALL_ENUM(UBlockCode, "SYRIAC_SUPPLEMENT", UBLOCK_SYRIAC_SUPPLEMENT);
+    INSTALL_ENUM(UBlockCode, "ZANABAZAR_SQUARE", UBLOCK_ZANABAZAR_SQUARE);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(62, 0, 0)
+    INSTALL_ENUM(UBlockCode, "CHESS_SYMBOLS", UBLOCK_CHESS_SYMBOLS);
+    INSTALL_ENUM(UBlockCode, "DOGRA", UBLOCK_DOGRA);
+    INSTALL_ENUM(UBlockCode, "GEORGIAN_EXTENDED", UBLOCK_GEORGIAN_EXTENDED);
+    INSTALL_ENUM(UBlockCode, "GUNJALA_GONDI", UBLOCK_GUNJALA_GONDI);
+    INSTALL_ENUM(UBlockCode, "HANIFI_ROHINGYA", UBLOCK_HANIFI_ROHINGYA);
+    INSTALL_ENUM(UBlockCode, "INDIC_SIYAQ_NUMBERS", UBLOCK_INDIC_SIYAQ_NUMBERS);
+    INSTALL_ENUM(UBlockCode, "MAKASAR", UBLOCK_MAKASAR);
+    INSTALL_ENUM(UBlockCode, "MAYAN_NUMERALS", UBLOCK_MAYAN_NUMERALS);
+    INSTALL_ENUM(UBlockCode, "MEDEFAIDRIN", UBLOCK_MEDEFAIDRIN);
+    INSTALL_ENUM(UBlockCode, "OLD_SOGDIAN", UBLOCK_OLD_SOGDIAN);
+    INSTALL_ENUM(UBlockCode, "SOGDIAN", UBLOCK_SOGDIAN);
 #endif
     INSTALL_ENUM(UBlockCode, "INVALID_CODE", UBLOCK_INVALID_CODE);
 
@@ -1303,4 +1419,219 @@ void _init_char(PyObject *m)
 
     INSTALL_ENUM(UPropertyNameChoice, "SHORT_PROPERTY_NAME", U_SHORT_PROPERTY_NAME);
     INSTALL_ENUM(UPropertyNameChoice, "LONG_PROPERTY_NAME", U_LONG_PROPERTY_NAME);
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(52, 0, 0)
+    INSTALL_ENUM(UBidiPairedBracketType, "NONE", U_BPT_NONE);
+    INSTALL_ENUM(UBidiPairedBracketType, "OPEN", U_BPT_OPEN);
+    INSTALL_ENUM(UBidiPairedBracketType, "CLOSE", U_BPT_CLOSE);
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    INSTALL_ENUM(UIndicPositionalCategory, "NA", U_INPC_NA);
+    INSTALL_ENUM(UIndicPositionalCategory, "BOTTOM", U_INPC_BOTTOM);
+    INSTALL_ENUM(UIndicPositionalCategory, "BOTTOM_AND_LEFT", U_INPC_BOTTOM_AND_LEFT);
+    INSTALL_ENUM(UIndicPositionalCategory, "BOTTOM_AND_RIGHT", U_INPC_BOTTOM_AND_RIGHT);
+    INSTALL_ENUM(UIndicPositionalCategory, "LEFT", U_INPC_LEFT);
+    INSTALL_ENUM(UIndicPositionalCategory, "LEFT_AND_RIGHT", U_INPC_LEFT_AND_RIGHT);
+    INSTALL_ENUM(UIndicPositionalCategory, "OVERSTRUCK", U_INPC_OVERSTRUCK);
+    INSTALL_ENUM(UIndicPositionalCategory, "RIGHT", U_INPC_RIGHT);
+    INSTALL_ENUM(UIndicPositionalCategory, "TOP", U_INPC_TOP);
+    INSTALL_ENUM(UIndicPositionalCategory, "TOP_AND_BOTTOM", U_INPC_TOP_AND_BOTTOM);
+    INSTALL_ENUM(UIndicPositionalCategory, "TOP_AND_BOTTOM_AND_RIGHT", U_INPC_TOP_AND_BOTTOM_AND_RIGHT);
+    INSTALL_ENUM(UIndicPositionalCategory, "TOP_AND_LEFT", U_INPC_TOP_AND_LEFT);
+    INSTALL_ENUM(UIndicPositionalCategory, "TOP_AND_LEFT_AND_RIGHT", U_INPC_TOP_AND_LEFT_AND_RIGHT);
+    INSTALL_ENUM(UIndicPositionalCategory, "TOP_AND_RIGHT", U_INPC_TOP_AND_RIGHT);
+    INSTALL_ENUM(UIndicPositionalCategory, "VISUAL_ORDER_LEFT", U_INPC_VISUAL_ORDER_LEFT);
+
+    INSTALL_ENUM(UIndicSyllabicCategory, "OTHER", U_INSC_OTHER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "AVAGRAHA", U_INSC_AVAGRAHA);
+    INSTALL_ENUM(UIndicSyllabicCategory, "BINDU", U_INSC_BINDU);
+    INSTALL_ENUM(UIndicSyllabicCategory, "BRAHMI_JOINING_NUMBER", U_INSC_BRAHMI_JOINING_NUMBER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CANTILLATION_MARK", U_INSC_CANTILLATION_MARK);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT", U_INSC_CONSONANT);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_DEAD", U_INSC_CONSONANT_DEAD);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_FINAL", U_INSC_CONSONANT_FINAL);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_HEAD_LETTER", U_INSC_CONSONANT_HEAD_LETTER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_INITIAL_POSTFIXED", U_INSC_CONSONANT_INITIAL_POSTFIXED);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_KILLER", U_INSC_CONSONANT_KILLER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_MEDIAL", U_INSC_CONSONANT_MEDIAL);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_PLACEHOLDER", U_INSC_CONSONANT_PLACEHOLDER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_PRECEDING_REPHA", U_INSC_CONSONANT_PRECEDING_REPHA);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_PREFIXED", U_INSC_CONSONANT_PREFIXED);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_SUBJOINED", U_INSC_CONSONANT_SUBJOINED);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_SUCCEEDING_REPHA", U_INSC_CONSONANT_SUCCEEDING_REPHA);
+    INSTALL_ENUM(UIndicSyllabicCategory, "CONSONANT_WITH_STACKER", U_INSC_CONSONANT_WITH_STACKER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "GEMINATION_MARK", U_INSC_GEMINATION_MARK);
+    INSTALL_ENUM(UIndicSyllabicCategory, "INVISIBLE_STACKER", U_INSC_INVISIBLE_STACKER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "JOINER", U_INSC_JOINER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "MODIFYING_LETTER", U_INSC_MODIFYING_LETTER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "NON_JOINER", U_INSC_NON_JOINER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "NUKTA", U_INSC_NUKTA);
+    INSTALL_ENUM(UIndicSyllabicCategory, "NUMBER", U_INSC_NUMBER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "NUMBER_JOINER", U_INSC_NUMBER_JOINER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "PURE_KILLER", U_INSC_PURE_KILLER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "REGISTER_SHIFTER", U_INSC_REGISTER_SHIFTER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "SYLLABLE_MODIFIER", U_INSC_SYLLABLE_MODIFIER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "TONE_LETTER", U_INSC_TONE_LETTER);
+    INSTALL_ENUM(UIndicSyllabicCategory, "TONE_MARK", U_INSC_TONE_MARK);
+    INSTALL_ENUM(UIndicSyllabicCategory, "VIRAMA", U_INSC_VIRAMA);
+    INSTALL_ENUM(UIndicSyllabicCategory, "VISARGA", U_INSC_VISARGA);
+    INSTALL_ENUM(UIndicSyllabicCategory, "VOWEL", U_INSC_VOWEL);
+    INSTALL_ENUM(UIndicSyllabicCategory, "VOWEL_DEPENDENT", U_INSC_VOWEL_DEPENDENT);
+    INSTALL_ENUM(UIndicSyllabicCategory, "VOWEL_INDEPENDENT", U_INSC_VOWEL_INDEPENDENT);
+
+    INSTALL_ENUM(UVerticalOrientation, "ROTATED", U_VO_ROTATED);
+    INSTALL_ENUM(UVerticalOrientation, "TRANSFORMED_ROTATED", U_VO_TRANSFORMED_ROTATED);
+    INSTALL_ENUM(UVerticalOrientation, "TRANSFORMED_UPRIGHT", U_VO_TRANSFORMED_UPRIGHT);
+    INSTALL_ENUM(UVerticalOrientation, "UPRIGHT", U_VO_UPRIGHT);
+#endif
+
+#if U_ICU_VERSION_HEX >= 0x04000000
+    INSTALL_ENUM(UWordBreakValues, "CR", U_WB_CR);
+    INSTALL_ENUM(UWordBreakValues, "EXTEND", U_WB_EXTEND);
+    INSTALL_ENUM(UWordBreakValues, "LF", U_WB_LF);
+    INSTALL_ENUM(UWordBreakValues, "MIDNUMLET", U_WB_MIDNUMLET);
+    INSTALL_ENUM(UWordBreakValues, "NEWLINE", U_WB_NEWLINE);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(50, 0, 0)
+    INSTALL_ENUM(UWordBreakValues, "REGIONAL_INDICATOR", U_WB_REGIONAL_INDICATOR);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(52, 0, 0)
+    INSTALL_ENUM(UWordBreakValues, "HEBREW_LETTER", U_WB_HEBREW_LETTER);
+    INSTALL_ENUM(UWordBreakValues, "SINGLE_QUOTE", U_WB_SINGLE_QUOTE);
+    INSTALL_ENUM(UWordBreakValues, "DOUBLE_QUOTE", U_WB_DOUBLE_QUOTE);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(58, 0, 0)
+    INSTALL_ENUM(UWordBreakValues, "E_BASE", U_WB_E_BASE);
+    INSTALL_ENUM(UWordBreakValues, "E_BASE_GAZ", U_WB_E_BASE_GAZ);
+    INSTALL_ENUM(UWordBreakValues, "E_MODIFIER", U_WB_E_MODIFIER);
+    INSTALL_ENUM(UWordBreakValues, "ZWJ", U_WB_ZWJ);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(62, 0, 0)
+    INSTALL_ENUM(UWordBreakValues, "WSEGSPACE", U_WB_WSEGSPACE);
+#endif
+
+#if U_ICU_VERSION_HEX >= 0x04060000
+    INSTALL_ENUM(UJoiningGroup, "TEH_MARBUTA_GOAL", U_JG_TEH_MARBUTA_GOAL);
+#endif
+    INSTALL_ENUM(UJoiningGroup, "FE", U_JG_FE);
+    INSTALL_ENUM(UJoiningGroup, "KHAPH", U_JG_KHAPH);
+    INSTALL_ENUM(UJoiningGroup, "ZHAIN", U_JG_ZHAIN);
+#if U_ICU_VERSION_HEX >= 0x04000000
+    INSTALL_ENUM(UJoiningGroup, "BURUSHASKI_YEH_BARREE", U_JG_BURUSHASKI_YEH_BARREE);
+#endif
+#if U_ICU_VERSION_HEX >= 0x04040000
+    INSTALL_ENUM(UJoiningGroup, "FARSI_YEH", U_JG_FARSI_YEH);
+    INSTALL_ENUM(UJoiningGroup, "NYA", U_JG_NYA);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(49, 0, 0)
+    INSTALL_ENUM(UJoiningGroup, "ROHINGYA_YEH", U_JG_ROHINGYA_YEH);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(54, 0, 0)
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_ALEPH", U_JG_MANICHAEAN_ALEPH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_AYIN", U_JG_MANICHAEAN_AYIN);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_BETH", U_JG_MANICHAEAN_BETH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_DALETH", U_JG_MANICHAEAN_DALETH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_DHAMEDH", U_JG_MANICHAEAN_DHAMEDH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_FIVE", U_JG_MANICHAEAN_FIVE);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_GIMEL", U_JG_MANICHAEAN_GIMEL);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_HETH", U_JG_MANICHAEAN_HETH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_HUNDRED", U_JG_MANICHAEAN_HUNDRED);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_KAPH", U_JG_MANICHAEAN_KAPH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_LAMEDH", U_JG_MANICHAEAN_LAMEDH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_MEM", U_JG_MANICHAEAN_MEM);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_NUN", U_JG_MANICHAEAN_NUN);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_ONE", U_JG_MANICHAEAN_ONE);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_PE", U_JG_MANICHAEAN_PE);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_QOPH", U_JG_MANICHAEAN_QOPH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_RESH", U_JG_MANICHAEAN_RESH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_SADHE", U_JG_MANICHAEAN_SADHE);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_SAMEKH", U_JG_MANICHAEAN_SAMEKH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_TAW", U_JG_MANICHAEAN_TAW);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_TEN", U_JG_MANICHAEAN_TEN);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_TETH", U_JG_MANICHAEAN_TETH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_THAMEDH", U_JG_MANICHAEAN_THAMEDH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_TWENTY", U_JG_MANICHAEAN_TWENTY);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_WAW", U_JG_MANICHAEAN_WAW);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_YODH", U_JG_MANICHAEAN_YODH);
+    INSTALL_ENUM(UJoiningGroup, "MANICHAEAN_ZAYIN", U_JG_MANICHAEAN_ZAYIN);
+    INSTALL_ENUM(UJoiningGroup, "STRAIGHT_WAW", U_JG_STRAIGHT_WAW);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(58, 0, 0)
+    INSTALL_ENUM(UJoiningGroup, "AFRICAN_FEH", U_JG_AFRICAN_FEH);
+    INSTALL_ENUM(UJoiningGroup, "AFRICAN_NOON", U_JG_AFRICAN_NOON);
+    INSTALL_ENUM(UJoiningGroup, "AFRICAN_QAF", U_JG_AFRICAN_QAF);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(60, 0, 0)
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_BHA", U_JG_MALAYALAM_BHA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_JA", U_JG_MALAYALAM_JA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_LLA", U_JG_MALAYALAM_LLA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_LLLA", U_JG_MALAYALAM_LLLA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_NGA", U_JG_MALAYALAM_NGA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_NNA", U_JG_MALAYALAM_NNA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_NNNA", U_JG_MALAYALAM_NNNA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_NYA", U_JG_MALAYALAM_NYA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_RA", U_JG_MALAYALAM_RA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_SSA", U_JG_MALAYALAM_SSA);
+    INSTALL_ENUM(UJoiningGroup, "MALAYALAM_TTA", U_JG_MALAYALAM_TTA);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(62, 0, 0)
+    INSTALL_ENUM(UJoiningGroup, "HANIFI_ROHINGYA_KINNA_YA", U_JG_HANIFI_ROHINGYA_KINNA_YA);
+    INSTALL_ENUM(UJoiningGroup, "HANIFI_ROHINGYA_PA", U_JG_HANIFI_ROHINGYA_PA);
+#endif
+
+    INSTALL_ENUM(ULineBreak, "INSEPARABLE", U_LB_INSEPARABLE);
+    INSTALL_ENUM(ULineBreak, "NEXT_LINE", U_LB_NEXT_LINE);
+    INSTALL_ENUM(ULineBreak, "WORD_JOINER", U_LB_WORD_JOINER);
+    INSTALL_ENUM(ULineBreak, "H2", U_LB_H2);
+    INSTALL_ENUM(ULineBreak, "H3", U_LB_H3);
+    INSTALL_ENUM(ULineBreak, "JL", U_LB_JL);
+    INSTALL_ENUM(ULineBreak, "JT", U_LB_JT);
+
+    INSTALL_ENUM(UGraphemeClusterBreak, "OTHER", U_GCB_OTHER);
+    INSTALL_ENUM(UGraphemeClusterBreak, "CONTROL", U_GCB_CONTROL);
+    INSTALL_ENUM(UGraphemeClusterBreak, "CR", U_GCB_CR);
+    INSTALL_ENUM(UGraphemeClusterBreak, "EXTEND", U_GCB_EXTEND);
+    INSTALL_ENUM(UGraphemeClusterBreak, "L", U_GCB_L);
+    INSTALL_ENUM(UGraphemeClusterBreak, "LF", U_GCB_LF);
+    INSTALL_ENUM(UGraphemeClusterBreak, "LV", U_GCB_LV);
+    INSTALL_ENUM(UGraphemeClusterBreak, "LVT", U_GCB_LVT);
+    INSTALL_ENUM(UGraphemeClusterBreak, "T", U_GCB_T);
+    INSTALL_ENUM(UGraphemeClusterBreak, "V", U_GCB_V);
+#if U_ICU_VERSION_HEX >= 0x04000000
+    INSTALL_ENUM(UGraphemeClusterBreak, "SPACING_MARK", U_GCB_SPACING_MARK);
+    INSTALL_ENUM(UGraphemeClusterBreak, "PREPEND", U_GCB_PREPEND);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(50, 0, 0)
+    INSTALL_ENUM(UGraphemeClusterBreak, "REGIONAL_INDICATOR", U_GCB_REGIONAL_INDICATOR);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(58, 0, 0)
+    INSTALL_ENUM(UGraphemeClusterBreak, "E_BASE", U_GCB_E_BASE);
+    INSTALL_ENUM(UGraphemeClusterBreak, "E_BASE_GAZ", U_GCB_E_BASE_GAZ);
+    INSTALL_ENUM(UGraphemeClusterBreak, "GLUE_AFTER_ZWJ", U_GCB_GLUE_AFTER_ZWJ);
+    INSTALL_ENUM(UGraphemeClusterBreak, "ZWJ", U_GCB_ZWJ);
+#endif
+
+    INSTALL_ENUM(UHangulSyllableType, "NOT_APPLICABLE", U_HST_NOT_APPLICABLE);
+    INSTALL_ENUM(UHangulSyllableType, "LEADING_JAMO", U_HST_LEADING_JAMO);
+    INSTALL_ENUM(UHangulSyllableType, "VOWEL_JAMO", U_HST_VOWEL_JAMO);
+    INSTALL_ENUM(UHangulSyllableType, "TRAILING_JAMO", U_HST_TRAILING_JAMO);
+    INSTALL_ENUM(UHangulSyllableType, "LV_SYLLABLE", U_HST_LV_SYLLABLE);
+    INSTALL_ENUM(UHangulSyllableType, "LVT_SYLLABLE", U_HST_LVT_SYLLABLE);
+
+#if U_ICU_VERSION_HEX >= 0x04040000
+    INSTALL_ENUM(ULineBreak, "CLOSE_PARENTHESIS", U_LB_CLOSE_PARENTHESIS);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(49, 0, 0)
+    INSTALL_ENUM(ULineBreak, "CONDITIONAL_JAPANESE_STARTER", U_LB_CONDITIONAL_JAPANESE_STARTER);
+    INSTALL_ENUM(ULineBreak, "HEBREW_LETTER", U_LB_HEBREW_LETTER);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(50, 0, 0)
+    INSTALL_ENUM(ULineBreak, "REGIONAL_INDICATOR", U_LB_REGIONAL_INDICATOR);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(58, 0, 0)
+    INSTALL_ENUM(ULineBreak, "E_BASE", U_LB_E_BASE);
+    INSTALL_ENUM(ULineBreak, "E_MODIFIER", U_LB_E_MODIFIER);
+    INSTALL_ENUM(ULineBreak, "ZWJ", U_LB_ZWJ);
+#endif
 }
