@@ -183,6 +183,7 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/tmunit.h>
 #include <unicode/tmutamt.h>
 #include <unicode/stringpiece.h>
+#include <unicode/numsys.h>
 #endif
 
 #if U_ICU_VERSION_HEX >= 0x04040000
@@ -194,6 +195,10 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <typeinfo>
 #endif
 
+#if U_ICU_VERSION_HEX >= 0x04080000
+#include <unicode/alphaindex.h>
+#endif
+
 #if U_ICU_VERSION_HEX >= VERSION_HEX(50, 0, 0)
 #include <unicode/listformatter.h>
 #endif
@@ -215,6 +220,7 @@ typedef intintobjargproc ssizessizeobjargproc;
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(59, 0, 0)
 #include <unicode/casemap.h>
+#include <unicode/edits.h>
 #endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(60, 0, 0)
