@@ -6,10 +6,10 @@
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions: 
+ * Software is furnished to do so, subject to the following conditions:
  *
  * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software. 
+ * in all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -112,6 +112,10 @@ static PyObject *t_locale_getCanadaFrench(PyTypeObject *type);
 static PyObject *t_locale_getDefault(PyTypeObject *type);
 static PyObject *t_locale_setDefault(PyTypeObject *type, PyObject *args);
 static PyObject *t_locale_createFromName(PyTypeObject *type, PyObject *args);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+static PyObject *t_locale_forLanguageTag(PyTypeObject *type, PyObject *arg);
+static PyObject *t_locale_toLanguageTag(t_locale *self);
+#endif
 static PyObject *t_locale_createCanonical(PyTypeObject *type, PyObject *arg);
 static PyObject *t_locale_getAvailableLocales(PyTypeObject *type);
 static PyObject *t_locale_getISOCountries(PyTypeObject *type);
@@ -172,6 +176,10 @@ static PyMethodDef t_locale_methods[] = {
     DECLARE_METHOD(t_locale, setDefault, METH_VARARGS | METH_CLASS),
     DECLARE_METHOD(t_locale, createFromName, METH_VARARGS | METH_CLASS),
     DECLARE_METHOD(t_locale, createCanonical, METH_O | METH_CLASS),
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    DECLARE_METHOD(t_locale, forLanguageTag, METH_O | METH_CLASS),
+    DECLARE_METHOD(t_locale, toLanguageTag, METH_NOARGS),
+#endif
     DECLARE_METHOD(t_locale, getAvailableLocales, METH_NOARGS | METH_CLASS),
     DECLARE_METHOD(t_locale, getISOCountries, METH_NOARGS | METH_CLASS),
     DECLARE_METHOD(t_locale, getISOLanguages, METH_NOARGS | METH_CLASS),
@@ -405,7 +413,7 @@ static int t_locale_init(t_locale *self, PyObject *args, PyObject *kwds)
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
     }
-        
+
     if (self->object)
         return 0;
 
@@ -899,6 +907,40 @@ static PyObject *t_locale_createCanonical(PyTypeObject *type, PyObject *arg)
     return PyErr_SetArgsError(type, "createCanonical", arg);
 }
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+
+static PyObject *t_locale_forLanguageTag(PyTypeObject *type, PyObject *arg)
+{
+    Locale locale;
+    charsArg name;
+
+    if (!parseArg(arg, "n", &name))
+    {
+        STATUS_CALL(locale = Locale::forLanguageTag(StringPiece(name), status));
+        return wrap_Locale(locale);
+    }
+
+    return PyErr_SetArgsError(type, "forLanguageTag", arg);
+}
+
+static PyObject *t_locale_toLanguageTag(t_locale *self)
+{
+    struct sink {
+        UnicodeString u;
+        void append(const char *data, int32_t n)
+        {
+            u.append(UnicodeString(data, n, US_INV));
+        }
+    } buffer;
+    StringByteSink<struct sink> sbs(&buffer);
+
+    STATUS_CALL(self->object->toLanguageTag(sbs, status));
+
+    return PyUnicode_FromUnicodeString(&buffer.u);
+}
+
+#endif
+
 static PyObject *t_locale_getAvailableLocales(PyTypeObject *type)
 {
     int count;
@@ -908,6 +950,7 @@ static PyObject *t_locale_getAvailableLocales(PyTypeObject *type)
     for (int32_t i = 0; i < count; i++) {
         Locale *locale = (Locale *) locales + i;
         PyObject *obj = wrap_Locale(locale, 0);
+
         PyDict_SetItemString(dict, locale->getName(), obj);
         Py_DECREF(obj);
     }
@@ -995,7 +1038,7 @@ static int t_resourcebundle_init(t_resourcebundle *self,
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
     }
-        
+
     if (self->object)
         return 0;
 
@@ -1427,7 +1470,7 @@ static int t_localedata_init(t_localedata *self, PyObject *args, PyObject *kwds)
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
     }
-        
+
     if (self->object)
         return 0;
 
@@ -1658,7 +1701,7 @@ static PyObject *t_region_getContainedRegions(t_region *self, PyObject *args)
         }
         break;
     }
-        
+
     return PyErr_SetArgsError((PyObject *) self, "getContainedRegions", args);
 }
 
