@@ -6,10 +6,10 @@
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions: 
+ * Software is furnished to do so, subject to the following conditions:
  *
  * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software. 
+ * in all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -38,9 +38,6 @@ DECLARE_CONSTANTS_TYPE(UScriptUsage);
 
 /* Script */
 
-struct UNone;
-typedef struct UNone UNone;
-
 class t_script : public _wrapper {
 public:
     UNone *object;
@@ -530,6 +527,45 @@ void _init_script(PyObject *m)
     INSTALL_ENUM(UScriptCode, "KHOJKI", USCRIPT_KHOJKI);
     INSTALL_ENUM(UScriptCode, "TIRHUTA", USCRIPT_TIRHUTA);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(52, 0, 0)
+    INSTALL_ENUM(UScriptCode, "MIAO", USCRIPT_MIAO);
+    INSTALL_ENUM(UScriptCode, "CAUCASIAN_ALBANIAN", USCRIPT_CAUCASIAN_ALBANIAN);
+    INSTALL_ENUM(UScriptCode, "MAHAJANI", USCRIPT_MAHAJANI);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(54, 0, 0)
+    INSTALL_ENUM(UScriptCode, "DUPLOYAN", USCRIPT_DUPLOYAN);
+    INSTALL_ENUM(UScriptCode, "KHUDAWADI", USCRIPT_KHUDAWADI);
+    INSTALL_ENUM(UScriptCode, "AHOM", USCRIPT_AHOM);
+    INSTALL_ENUM(UScriptCode, "HATRAN", USCRIPT_HATRAN);
+    INSTALL_ENUM(UScriptCode, "MODI", USCRIPT_MODI);
+    INSTALL_ENUM(UScriptCode, "MULTANI", USCRIPT_MULTANI);
+    INSTALL_ENUM(UScriptCode, "PAU_CIN_HAU", USCRIPT_PAU_CIN_HAU);
+    INSTALL_ENUM(UScriptCode, "SIDDHAM", USCRIPT_SIDDHAM);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(58, 0, 0)
+    INSTALL_ENUM(UScriptCode, "ADLAM", USCRIPT_ADLAM);
+    INSTALL_ENUM(UScriptCode, "BHAIKSUKI", USCRIPT_BHAIKSUKI);
+    INSTALL_ENUM(UScriptCode, "MARCHEN", USCRIPT_MARCHEN);
+    INSTALL_ENUM(UScriptCode, "NEWA", USCRIPT_NEWA);
+    INSTALL_ENUM(UScriptCode, "OSAGE", USCRIPT_OSAGE);
+    INSTALL_ENUM(UScriptCode, "HAN_WITH_BOPOMOFO", USCRIPT_HAN_WITH_BOPOMOFO);
+    INSTALL_ENUM(UScriptCode, "JAMO", USCRIPT_JAMO);
+    INSTALL_ENUM(UScriptCode, "SYMBOLS_EMOJI", USCRIPT_SYMBOLS_EMOJI);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(60, 0, 0)
+    INSTALL_ENUM(UScriptCode, "MASARAM_GONDI", USCRIPT_MASARAM_GONDI);
+    INSTALL_ENUM(UScriptCode, "SOYOMBO", USCRIPT_SOYOMBO);
+    INSTALL_ENUM(UScriptCode, "ZANABAZAR_SQUARE", USCRIPT_ZANABAZAR_SQUARE);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(62, 0, 0)
+    INSTALL_ENUM(UScriptCode, "DOGRA", USCRIPT_DOGRA);
+    INSTALL_ENUM(UScriptCode, "GUNJALA_GONDI", USCRIPT_GUNJALA_GONDI);
+    INSTALL_ENUM(UScriptCode, "MAKASAR", USCRIPT_MAKASAR);
+    INSTALL_ENUM(UScriptCode, "MEDEFAIDRIN", USCRIPT_MEDEFAIDRIN);
+    INSTALL_ENUM(UScriptCode, "HANIFI_ROHINGYA", USCRIPT_HANIFI_ROHINGYA);
+    INSTALL_ENUM(UScriptCode, "SOGDIAN", USCRIPT_SOGDIAN);
+    INSTALL_ENUM(UScriptCode, "OLD_SOGDIAN", USCRIPT_OLD_SOGDIAN);
+#endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
     INSTALL_ENUM(UScriptUsage, "NOT_ENCODED", USCRIPT_USAGE_NOT_ENCODED);
